"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.QiankunVue2Alive = void 0;
var _interceptor = require("./interceptor");
var _cache = require("./cache");
var _utils = require("./utils");
var _strategy = require("./strategy");
/// <reference path="./types.js" />

var QiankunVue2Alive = exports.QiankunVue2Alive = {
  name: 'qiankun-vue2-alive',
  "abstract": true,
  props: {
    current: Array,
    blacklist: [Array, RegExp, Function],
    max: [String, Number]
  },
  methods: {
    cacheVNode: function cacheVNode() {
      var cache = this.cache,
        vnodeToCache = this.vnodeToCache,
        keyToCache = this.keyToCache;
      if (vnodeToCache) {
        var componentInstance = vnodeToCache.componentInstance,
          tag = vnodeToCache.tag;
        var hasKey = !!cache.get(keyToCache);
        cache.set(keyToCache, {
          componentInstance: componentInstance,
          tag: tag
        });

        // prune oldest entry
        var max = this.max && parseInt(this.max);
        if (max && max > 0 && cache.size > max) {
          (0, _cache.pruneCacheEntry)(cache, cache.keys().next().value, this._vnode);
        }
        if (!hasKey) {
          this.$emit('update', (0, _cache.cacheMapToArray)(cache));
        }
        this.vnodeToCache = null;
      }
    }
  },
  created: function created() {
    /** @type {CacheEntryMap} */
    this.cache = new Map();
  },
  destroyed: function destroyed() {
    for (var key in this.cache.keys()) {
      (0, _cache.pruneCacheEntry)(this.cache, key);
    }
  },
  mounted: function mounted() {
    var _this = this;
    this.$watch('blacklist', function (val) {
      (0, _cache.pruneCache)(_this, function (key) {
        return (0, _strategy.hitBlacklist)({
          blacklist: val
        }, key);
      });
    });
    this.$watch('current', function (val) {
      (0, _cache.pruneDeltaCache)(_this, val);
    });
    this.cacheVNode();
  },
  updated: function updated() {
    this.cacheVNode();
  },
  render: function render() {
    var slot = this.$slots["default"];
    var vnode = (0, _utils.getFirstComponentChild)(slot);
    var componentOptions = vnode && vnode.componentOptions;
    if (componentOptions) {
      var key = vnode.key;
      if (!(0, _interceptor.renderIntercepor)(this, key)) {
        return vnode;
      }

      /**
       * @type {CacheEntryMap}
       */
      var cache = this.cache;
      if (cache.get(key)) {
        var cacheEntry = cache.get(key);
        vnode.componentInstance = cacheEntry.componentInstance;
        // make current key freshest
        cache["delete"](key);
        cache.set(key, cacheEntry);
      } else {
        // delay setting the cache until update
        this.vnodeToCache = vnode;
        this.keyToCache = key;
      }
      vnode.data.keepAlive = true;
    }
    return vnode || slot && slot[0];
  }
};