"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFirstComponentChild = getFirstComponentChild;
exports.getVNodeKey = getVNodeKey;
exports.isArray = isArray;
exports.isAsyncPlaceholder = isAsyncPlaceholder;
exports.isDef = isDef;
exports.isFunction = isFunction;
exports.isRegExp = isRegExp;
exports.matches = matches;
/// <reference path="./types.js" />

var objToString = Object.prototype.toString;
function isFunction(obj) {
  return typeof obj === 'function';
}
function isRegExp(obj) {
  return objToString.call(obj) === '[object RegExp]';
}
function isArray(obj) {
  return objToString.call(obj) === '[object Array]';
}

/**
 * @param {import('vue').VNode} node
 * @returns {boolean}
 */
function isAsyncPlaceholder(node) {
  return node.isComment && node.asyncFactory;
}
function isDef(v) {
  return v !== undefined && v !== null;
}

/**
 * @param {import('vue').VNode[]} [children]
 * @returns {VNode|undefined}
 */
function getFirstComponentChild(children) {
  if (isArray(children)) {
    for (var i = 0; i < children.length; i++) {
      var c = children[i];
      if (isDef(c) && (isDef(c.componentOptions) || isAsyncPlaceholder(c))) {
        return c;
      }
    }
  }
}

/**
 * @param {string[]} pattern
 * @param {string} key
 * @returns {boolean}
 */
function matches(pattern, key) {
  if (isArray(pattern)) {
    return pattern.indexOf(key) > -1;
  }

  /* istanbul ignore next */
  return false;
}
function getVNodeKey(vm) {
  var _vm$_vnode;
  return (_vm$_vnode = vm._vnode) === null || _vm$_vnode === void 0 ? void 0 : _vm$_vnode.key;
}